var Stats = function (show, reportDuration) {

	var reporter = new Reporter(reportDuration); // 上报器

	var isShowPanel = show; // true 显示面板；false 不显示面板
	var mode = 0;

	var container = DragBox();
	container.addEventListener('click', function (event) {

        event.preventDefault();
        showPanel(++mode % container.children.length);

    }, false);


	//
	function addPanel(panel) {
        //if(container != null) {
            container.appendChild(panel.dom);
        //}

		return panel;

	}

	function showPanel(id) {

        //if (container != null) {}
            for (var i = 0; i < container.children.length; i++) {

                container.children[i].style.display = i === id ? 'block' : 'none';

            }
		//}

		mode = id;
	}

	function updatePanel(panel, value, maxValue) {
		if (panel != null) {
			panel.update(value, maxValue);
		}
	}

	var beginTime = (performance || Date).now(),
		prevTime = beginTime,
		frames = 0;

	var fpsPanel;
	var msPanel;
	var memPanel;
	var memoryEnable = false
	if (self.performance && self.performance.memory) {
		memoryEnable = true;
	}


	// 要显示面板的时候才添加进来
	if (isShowPanel == true) {

		fpsPanel = addPanel(new Stats.Panel('FPS', '#0ff', '#002'));
		msPanel = addPanel(new Stats.Panel('MS', '#0f0', '#020'));

		if (memoryEnable == true) {
			memPanel = addPanel(new Stats.Panel('MB', '#f08', '#201'));
		}

		showPanel(0);
	}

	return {

		REVISION: 16,

		dom: container,

		addPanel: addPanel,
		showPanel: showPanel,

		begin: function () {

			beginTime = (performance || Date).now();

		},

		end: function () {

			frames++;

			var time = (performance || Date).now();
			var ms = time - beginTime // 每帧的绘制时间
			updatePanel(msPanel, ms, 200)

			if (time >= prevTime + 1000) {

				var fps = (frames * 1000) / (time - prevTime); // 帧数
				updatePanel(fpsPanel, fps, 100)

				// 帧数、每帧绘制时间上报给客户端
				var reportJson = {fps: fps.toFixed(3), ms: ms.toFixed(3)}

				prevTime = time;
				frames = 0;

				if (memoryEnable == true) {

					var memory = performance.memory;
					var usedJSHeapSize = memory.usedJSHeapSize / 1048576; // js 已使用的堆大小
					var jsHeapSizeLimit = memory.jsHeapSizeLimit / 1048576 // js 大小限制

					updatePanel(memPanel, usedJSHeapSize, jsHeapSizeLimit)

					// JS heap limit 及 used 上报
					reportJson.usedJSHeapSize = usedJSHeapSize.toFixed(3)
					reportJson.jsHeapSizeLimit = jsHeapSizeLimit.toFixed(3)
				}

				reporter.reportPerformance(reportJson);
			}

			return time;

		},

		update: function () {

			beginTime = this.end();

		},


		// Backwards Compatibility
		domElement: container,
		setMode: showPanel

	};

};

Stats.Panel = function (name, fg, bg) {

	var min = Infinity,
		max = 0,
		round = Math.round;
	var PR = round(window.devicePixelRatio || 1);

	var WIDTH = 80 * PR,
		HEIGHT = 48 * PR,
		TEXT_X = 3 * PR,
		TEXT_Y = 2 * PR,
		GRAPH_X = 3 * PR,
		GRAPH_Y = 15 * PR,
		GRAPH_WIDTH = 74 * PR,
		GRAPH_HEIGHT = 30 * PR;

	var canvas = document.createElement('canvas');
	canvas.width = WIDTH;
	canvas.height = HEIGHT;
	canvas.style.cssText = 'width:100px;height:48px;visibility:visible';

	var context = canvas.getContext('2d');
	context.font = 'bold ' + (9 * PR) + 'px Helvetica,Arial,sans-serif';
	context.textBaseline = 'top';

	context.fillStyle = bg;
	context.fillRect(0, 0, WIDTH, HEIGHT);

	context.fillStyle = fg;
	context.fillText(name, TEXT_X, TEXT_Y);
	context.fillRect(GRAPH_X, GRAPH_Y, GRAPH_WIDTH, GRAPH_HEIGHT);

	context.fillStyle = bg;
	context.globalAlpha = 0.9;
	context.fillRect(GRAPH_X, GRAPH_Y, GRAPH_WIDTH, GRAPH_HEIGHT);

	var counter = 0;
	return {

		dom: canvas,

		update: function (value, maxValue) {

			min = Math.min(min, value);
			max = Math.max(max, value);

			context.fillStyle = bg;
			context.globalAlpha = 1;
			context.fillRect(0, 0, WIDTH, GRAPH_Y);
			context.fillStyle = fg;
			context.fillText(value.toFixed(2) + ' ' + name + ' (' + round(min) + '-' + round(max) + ')', TEXT_X, TEXT_Y);

			context.drawImage(canvas, GRAPH_X + PR, GRAPH_Y, GRAPH_WIDTH - PR, GRAPH_HEIGHT, GRAPH_X, GRAPH_Y, GRAPH_WIDTH - PR, GRAPH_HEIGHT);

			context.fillRect(GRAPH_X + GRAPH_WIDTH - PR, GRAPH_Y, PR, GRAPH_HEIGHT);

			context.fillStyle = bg;
			context.globalAlpha = 0.9;
			context.fillRect(GRAPH_X + GRAPH_WIDTH - PR, GRAPH_Y, PR, round((1 - (value / maxValue)) * GRAPH_HEIGHT));
		}

	};
};

var Reporter = function (reportDuration) {

	if (reportDuration <= 0) {
		reportDuration = 10 // 默认十秒钟
	}

	var reportCount = reportDuration; // reportDuration 单位为秒，每秒刷新一次帧数
	var fpsCountor = 0; // 每刷新一次帧数计数一次

	return {
		reportPerformance: function (data) {
			fpsCountor++;

			if (fpsCountor >= reportCount) {
				fpsCountor = 0;

				if ((window.PlayMonitor) && typeof window.PlayMonitor.reportPerformance != 'undefined') {
				    var json = JSON.stringify(data)
					PlayMonitor.reportPerformance(json);
				} else if (window.webkit && window.webkit.messageHandlers && window.webkit.messageHandlers.reportPerformance) {
					window.webkit.messageHandlers.reportPerformance.postMessage(data);
				}
			}
		}
	}
}


var DragBox = function () {

	var container = document.createElement('div');
	container.style.cssText = 'position:absolute;top:20px;left:50px;cursor:pointer;opacity:0.9;z-index:10000';

	container.addEventListener('touchstart', function (e) {
		var ev = e || window.event;
		var touch = ev.targetTouches[0];
		oL = touch.clientX - container.offsetLeft;
		oT = touch.clientY - container.offsetTop;
		document.addEventListener("touchmove", defaultEvent, false);
	})

	container.addEventListener('touchmove', function (e) {
		var ev = e || window.event;
		var touch = ev.targetTouches[0];
		var oLeft = touch.clientX - oL;
		var oTop = touch.clientY - oT;
		//

		container.style.left = oLeft + 'px';
		container.style.top = oTop + 'px';

	})

	container.addEventListener('touchend', function () {
		document.removeEventListener("touchmove", defaultEvent);
	})

	function defaultEvent(e) {


		e.preventDefault();
	}

	return container
}


function monitor(show, reportDuration) {
	var stats = new Stats(show, reportDuration);

	(show == true) && document.body.appendChild(stats.dom);
	requestAnimationFrame(function loop() {
		stats.update();

		requestAnimationFrame(loop)
	});
}
